
/* Copyright 2013 Buddy Platform, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function(root) {

	var creds;
	
	function ensureInitialized() {
		if (!creds) {
			 return new Error("Buddy.init must be called before making service calls.");
		}

	}

	var knownErrors = {
 
        WrongSocketLoginOrPass:true, SecurityTokenInvalidPleaseRenew:true, SecurityTokenRenewed:true, SecurityTokenCouldNotBeRenewed:true, SecurityFailedBannedDeviceID:true,
        SecurityFailedBadUserNameOrPassword:true, SecurityFailedBadUserName:true, SecurityFailedBadUserPassword:true, DeviceIDAlreadyInSystem:true, UserNameAlreadyInUse:true,
        UserAccountNotFound:true, UserInvalidAccountSetting:true, UserAccountErrorSettingMetaValue:true, UserErrorUpdatingAccount:true, UserEmailTaken:true,
        UserProfileIDEmpty:true, IdentityValueEmpty:true, DeviceIDNotFound:true, DateTimeFormatWasIncorrect:true, LatLongFormatWasIncorrect:true,
        GeoLocationCategoryIncorrect:true, BadGeoLocationName:true, GeoLocationIDIncorrect:true, BadParameter:true, PhotoUploadGenericError:true, CouldNotFindPhotoTodelete:true,
        CouldNotDeleteFileGenericError:true, PhotoAlbumDoesNotExist:true, AlbumNamesCannotBeBlank:true, PhotoIDDoesNotExistInContext:true, dupelocation:true, invalidflagreason:true,
        EmptyDeviceURI:true, EmptyGroupName:true, EmptyImageURI:true, EmptyMessageCount:true, EmptyMessageTitle:true, EmptyRawMessage:true, EmptyToastTitle:true, EmptyToastSubTitle:true,
        EmptyToastParameter:true, GroupNameCannotBeEmpty:true, GroupSecurityCanOnlyBy0or1:true, GroupAlreadyExists:true, GroupChatIDEmpty:true, GroupChatNotFound:true, GroupOwnerSecurityError:true,
        ApplicationAPICallDisabledByDeveloper:true, ServiceErrorNull:true, ServiceErrorNegativeOne:true, UnknownServiceError:true

	}

	function processBuddyResult(data, callback) {


        if (data === "-1" || !data) {
            callback(new Error());
        }
        else if (typeof data == 'number' || (data.match && data.match(/^\d+$/)))
        {
            callback(null, new Number(data));
        }
        else if  (typeof data === 'object') {
        	callback(null, data);
        }
        else if (knownErrors[data]){
        	callback(new Error(data));
        }
        else {
        	callback(null, data);
        }
	}

	
	root.Buddy = {
		url: "https://webservice.buddyplatform.com/v1/",
		init: function(buddyAppName, buddyAppPassword) {

			if (!buddyAppName || !buddyAppPassword) throw new Error("buddyAppName and buddyAppPassword required.");

			if (!jQuery) {
				throw new Error("Buddy requires jQuery, which is not loaded.");
			}

			creds = {
				BuddyApplicationName: buddyAppName,
				BuddyApplicationPassword: buddyAppPassword
			};
		},
		callMethod: function(methodName, params, callback) {
			var err = ensureInitialized();

			// build the url.
			//
			if (typeof params === 'function') {
				callback = params;
				params = {};
			}

			if (err) {
				return callback && callback(err);
			}
			
			params = $.extend({}, creds, params);

			var formData = new FormData();

			for (var p in params) {
				formData.append(p, params[p]);
			}

			$.ajax({
				url:  root.Buddy.url + methodName,
				type: "POST",
				data: formData,
				contentType: false,
				processData: false,
				// headers: {
				// 	"x-buddy-sdk": "JS,v0.1"
				// },
				success: function(data) {
					callback && processBuddyResult(data, callback);
				},
				error: function(xhr, textStatus, textError) {
					if ("parsererror" == textStatus && xhr.status == 200) {
						callback && processBuddyResult(xhr.responseText, callback);
					}
					else {
						callback && callback(xhr);
					}
				}
			})
		}

	}




})(window);
